import bpy


def add_homa_house(length, width):
    """
    This function takes inputs and returns vertex and face arrays.
    no actual mesh data creation is done here.
    """

    vertices = [0.0, 0.0, 0.0,
                1.0, 0.0, 0.0,
                1.0, -0.5, 0.0,
                0.0, -0.5, 0.0,               
                1.0, 0.5, 0.0,
                0.0, 0.5, 0.0,
                ]
    edges = [0, 1,
            5, 4,
            3, 2,
            5, 0,
            3, 0,
            4, 1,
            2, 1,
            ]

    faces = [
            0, 1, 2, 3,
            0, 1, 4, 5
            ]

    # apply size
    for i in range(0, len(vertices), 3):
        vertices[i] *= length 
        vertices[i + 1] *= width


    return vertices, faces, edges


from bpy.props import FloatProperty, IntProperty, BoolProperty, FloatVectorProperty


class AddHomaHouse(bpy.types.Operator):
    '''Add a simple house mesh'''
    bl_idname = "mesh.primitive_homa_house_add"
    bl_label = "House"
    bl_options = {'REGISTER', 'UNDO'}

    width = FloatProperty(
            name="House width",
            description="House width",
            min=0.0, max=100.0,
            default=5.0,
            )
    length = FloatProperty(
            name="House length",
            description="House length",
            min=0.0, max=100.0,
            default=10.0,
            )
    foundation_height = FloatProperty(
            name="Foundation thickness",
            description="Foundation Height",
            min=0.0, max=100.0,
            default=0.0,
            )
    ground_floor_height = FloatProperty(
            name="Ground floor height",
            description="Ground Floor Height",
            min=0.0, max=100.0,
            default=2.5,
            )
    floors_thickness = FloatProperty(
            name="Floors thickness",
            description="Floors thickness usually 0.3",
            min=0.0, max=100.0,
            default=0.0,
            )
    first_floor_height = FloatProperty(
            name="First floor height",
            description="House Height",
            min=0.0, max=100.0,
            default=0.0,
            )
    last_floor_height = FloatProperty(
            name="Last floor height",
            description="Last floor height",
            min=0.0, max=100.0,
            default=0.0,
            )
    floors_number = IntProperty(
            name="Number of floors",
            description="Number of floors",
            min=1, max=100,
            default=0,
            )            
    generic_floor_height = FloatProperty(
            name="Generic floor height",
            description="Second floor height",
            min=0.0, max=100.0,
            default=0.0,
            )
    cornice_thickness = FloatProperty(
            name="Cornice thickness",
            description="Last chance to tune the cornice",
            min=0.0, max=100.0,
            default=0.0,
            )
    ridge_height = FloatProperty(
            name="Ridge height",
            description="Second floor height",
            min=0.0, max=100.0,
            default=3.0,
            )

                                                                                                                  

    # generic transform props
    view_align = BoolProperty(
            name="Align to View",
            default=False,
            )
    location = FloatVectorProperty(
            name="Location",
            subtype='TRANSLATION',
            )
    rotation = FloatVectorProperty(
            name="Rotation",
            subtype='EULER',
            )

    def execute(self, context):
        #   creating the basis shape...
        verts_loc, faces , edges= add_homa_house(self.length,                                     
                                     self.width,
                                     )

        mesh = bpy.data.meshes.new("Homa_House")

        mesh.vertices.add(len(verts_loc) // 3)
        mesh.edges.add(len(edges) // 2)
        mesh.faces.add(len(faces) // 4)      
        

        mesh.vertices.foreach_set("co", verts_loc)
        mesh.edges.foreach_set("vertices", edges)
        mesh.faces.foreach_set("vertices_raw", faces)
        mesh.update()
        #   the mesh is created let's transform it into house...

        
        
        
        #   3 murs etages eventuels 
        #   4 toiture
        
        
        
        
        
           

        # add the mesh as an object into the scene with this utility module
        from bpy_extras import object_utils        
        object_utils.object_data_add(context, mesh, operator=self)
        
        
        
        # prepare selections and variables before to start geometry
        obj=bpy.context.active_object
        current_total_edges=0
        nb_vertices_base=0
        base_edges=[]
        ridge_vertices=[] 
        
        # add custom properties for later I guess        
        obj['House_Width']= self.ground_floor_height     

        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.flip_normals()
        bpy.context.tool_settings.mesh_select_mode = [False, True, False]        
        bpy.ops.mesh.select_all(action='DESELECT')

        
        bpy.ops.object.mode_set(mode='OBJECT')
        for i in obj.data.edges:
            if i.index !=0 :
                i.select=True
                base_edges.append(i)
                nb_vertices_base=nb_vertices_base+1

            
        current_total_edges=current_total_edges+nb_vertices_base 
        bpy.ops.object.mode_set(mode='EDIT')
        
        bpy.ops.object.vertex_group_assign(new=True)
        obj.vertex_groups['Group'].name='Base'
        
        
        #   1 dalle
        if self.foundation_height > 0:
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.foundation_height), "constraint_axis":(False, False, True)})
            current_total_edges=current_total_edges+nb_vertices_base   
        #   2 murs rez
        if self.ground_floor_height > 0:
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.ground_floor_height), "constraint_axis":(False, False, True)})
            current_total_edges=current_total_edges+nb_vertices_base     
        #   3 murs etages eventuels
        # 1er etage

        if self.first_floor_height > 0:
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.floors_thickness), "constraint_axis":(False, False, True)})  
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.first_floor_height), "constraint_axis":(False, False, True)})          
            current_total_edges=current_total_edges+nb_vertices_base*2                       
        
        # plus d'etages        
        if self.generic_floor_height > 0:

            for i in range(self.floors_number):
                bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.floors_thickness), "constraint_axis":(False, False, True)})  
                bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.generic_floor_height), "constraint_axis":(False, False, True)})  
                current_total_edges=current_total_edges+nb_vertices_base*2
        # dernier etage
        if self.last_floor_height > 0:
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.floors_thickness), "constraint_axis":(False, False, True)})  
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.last_floor_height), "constraint_axis":(False, False, True)})       
            current_total_edges=current_total_edges+nb_vertices_base*2 
        #   4 corniche
        if self.cornice_thickness > 0:  
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.cornice_thickness), "constraint_axis":(False, False, True)})        
            current_total_edges=current_total_edges+nb_vertices_base 
        
        #   5 toiture
        if self.ridge_height > 0:  
            bpy.ops.mesh.extrude_edges_move(TRANSFORM_OT_translate={"value":(0, 0, self.ridge_height), "constraint_axis":(False, False, True)})         
            bpy.ops.object.vertex_group_assign(new=True)            
            obj.vertex_groups['Group'].name="Ridge"

            current_total_edges=current_total_edges+nb_vertices_base 
            bpy.context.tool_settings.mesh_select_mode = [True, False, False]
            bpy.ops.object.mode_set(mode='OBJECT')

            for i in obj.data.vertices:
                if i.select == True:
                    if i.co[0]==0:
                        i.select = False
                    
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.object.vertex_group_assign(new=True)            
            obj.vertex_groups['Group'].name="2nd_sprocket"
            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.vertex_group_set_active(group='Ridge')
            bpy.ops.object.vertex_group_select()
            bpy.ops.object.mode_set(mode='OBJECT')
            
            for i in obj.data.vertices:
                if i.select == True:
                    if i.co[0]==self.length:
                        i.select = False
                        
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.object.vertex_group_assign(new=True)            
            obj.vertex_groups['Group'].name="Main_sprocket"
            bpy.ops.mesh.merge(type='CENTER', uvs=False)
            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.vertex_group_set_active(group='2nd_sprocket')
            bpy.ops.object.vertex_group_select()
            bpy.ops.mesh.merge(type='CENTER', uvs=False)
            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.mode_set(mode='OBJECT')

                    
                  
            print(ridge_vertices)

            bpy.ops.object.mode_set(mode='EDIT')
    
        bpy.ops.object.mode_set(mode='OBJECT')
        obj.data.show_all_edges=True
        obj.show_wire=True
        return {'FINISHED'}


def menu_func(self, context):
    self.layout.operator(AddHomaHouse.bl_idname, icon='MESH_CUBE')


def register():
    bpy.utils.register_class(AddHomaHouse)
    bpy.types.INFO_MT_mesh_add.append(menu_func)


def unregister():
    bpy.utils.unregister_class(AddHomaHouse)
    bpy.types.INFO_MT_mesh_add.remove(menu_func)

if __name__ == "__main__":
    register()

    # test call
    bpy.ops.mesh.primitive_homa_house_add()
